#include "espacevectoriel.h"

ostream& operator<<(ostream& os,EspaceVectoriel &ev){
  if(ev.dim==0){
    os<<"<>";
    return os;
  }
  usint n=ev.base[0].get_nb_lignes();
  for(usint i=1;i<=n;i++){
    os<<"<";
    for(usint j=0;j<ev.dim;j++){
      os<<"["<<(ev.base[j])[i][1]<<"]\t";
    }
    os<<">"<<endl;
  }
  return os;
}

EspaceVectoriel resoud(const Matrice& A){
   Matrice B=A.Gauss();
  Matrice C;
  //Initialisation de l’espace vectoriel solution S
  EspaceVectoriel S;
  usint n=B.get_nb_colonnes();
  usint r=B.rang();
  S.dim=n-r;//dimension de S
  S.base=new Matrice[S.dim];
  S.base[0]=Matrice(n,1);
  Matrice b(n,1);
  for(usint i=0;i<S.dim;i++){
    S.base[i]=b;//initialisation des vecteurs de base a` 0^n
  }
  //Creation du tableau param
  usint* param=new usint[n+1]; //indice utile de 1 jusqu'à n
  usint* pivot=new usint[r+1]; //indice utile de 1 jusqu'à r
  usint l=1;
  usint np=1;
  for(usint j=1;j<=n;j++){
    if(B[l][j]==1){//pivot trouve
      param[j]=0;
      pivot[l]=j;//position pivolt ligne l
      l++;
    }
    else{
      param[j]=np;
      (S.base[np-1])[j][1]=1;
      np++;
    }
  }
  //Suite et fin du remplissage des vecteurs de bases de S
  for(usint i=1;i<=r;i++){
    for(usint j=pivot[i]+1;j<=n;j++){
      if(param[j]!=0){
	(S.base[param[j]-1])[pivot[i]][1]=-B[i][j];
      }
    }
  }
  return S;
}
  
EspaceVectoriel get_espace_vectoriel(Matrice* F,usint k){
  EspaceVectoriel R;
  if(k==0){
    R.dim=0;
    return R;
  }
  usint n=F[0].get_nb_lignes();
  Matrice A(n,k);
  for(usint j=1;j<=k;j++){
    if(F[j-1].get_nb_lignes()!=n){
      throw("Famille non définie");
    }
    for(usint i=1;i<=n;i++){
      A[i][j]=(F[j-1])[i][1];
    }
  }
  Matrice B=A.Gauss();
  R.dim=B.rang();
  R.base=new Matrice[R.dim];
  usint l=1;
  for(usint j=1;j<=k;j++){
    if(B[l][j]==1){
      R.base[l-1]=F[j-1];
      l++;
    }
  }
  return R;
}

EspaceVectoriel somme(EspaceVectoriel E,EspaceVectoriel F){
  if(E.dim==0){
    return F;
  }
  if(F.dim==0){
    return E;
  }
  usint n=E.base[0].get_nb_lignes();
  if(F.base[0].get_nb_lignes()!=n){
    throw("Espaces vectoriels incompatibles");
  }
  Matrice* G=new Matrice[E.dim+F.dim];
  for(usint i=0;i<E.dim;i++){
    G[i]=E.base[i];
  }
  for(usint i=0;i<F.dim;i++){
    G[i+E.dim]=F.base[i];
  }
  return get_espace_vectoriel(G,E.dim+F.dim);
}


EspaceVectoriel intersection(EspaceVectoriel E,EspaceVectoriel F){
  if(E.dim==0 or F.dim==0){
    EspaceVectoriel R;
    R.dim=0;
    return R;
  }
  usint n=E.base[0].get_nb_lignes();
  if(F.base[0].get_nb_lignes()!=n){
    throw("Espaces vectoriels incompatibles");
  }
  Matrice A(n,E.dim+F.dim);
  for(usint i=1;i<=n;i++){
    for(usint j=1;j<=E.dim;j++){
      A[i][j]=(E.base[j-1])[i][1];
    }
    for(usint j=1;j<=F.dim;j++){
      A[i][j+E.dim]=-(F.base[j-1])[i][1];
    }
  }
  EspaceVectoriel S=resoud(A);
  EspaceVectoriel R;
  R.dim=S.dim;
  R.base=new Matrice[R.dim];
  for(usint k=0;k<R.dim;k++){
    R.base[k]=Matrice(n,1);
    for(usint i=1;i<=E.dim;i++){
      R.base[k]=R.base[k]+(S.base[k])[i][1]*E.base[i-1];
    }
  }
  return R;
}
